@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2005  Mattia Dongili <malattia@linux.it>
+ *  Copyright (C) 2002-2006  Mattia Dongili <malattia@linux.it>
  *                           George Staikos <staikos@0wned.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <time.h>
 #include <unistd.h>
 #include "config_parser.h"
 #include "cpufreq_utils.h"
@@ -54,11 +55,18 @@ do { \
 	} \
 } while (0);
 
+static struct cpufreqd_info *cpufreqd_info;
+static struct rule *current_rule;
+static struct profile *current_profile;
+static int force_reinit = 0;
+static int force_exit = 0;
+static int timer_expired = 1; /* expired in order to run on the first loop */
+static int cpufreqd_mode = MODE_DYNAMIC; /* operation mode (manual / dynamic) */
+
 /* default configuration */
-struct cpufreqd_conf configuration = {
+static struct cpufreqd_conf default_configuration = {
 	.config_file		= CPUFREQD_CONFDIR"cpufreqd.conf",
 	.pidfile		= CPUFREQD_STATEDIR"cpufreqd.pid",
-	.cpu_num		= 1,
 	.poll_intv		= { .tv_sec = DEFAULT_POLL, .tv_usec = 0 },
 	.has_sysfs		= 1,
 	.no_daemon		= 0,
@@ -68,35 +76,70 @@ struct cpufreqd_conf configuration = {
 	.remote_gid		= 0,
 	.double_check		= 0,
 	.print_help		= 0,
-	.print_version		= 0
+	.print_version		= 0,
 };
+struct cpufreqd_conf *configuration;
+
+/*
+ * Try to read current kernel version.
+ */
+static int get_kversion(void) {
+	FILE *fp;
+	char kver[256];
+	int f = 0;
+
+	fp = fopen ("/proc/version", "r");
+	if (!fp) {
+		clog(LOG_ERR, "/proc/version: %s\n", strerror(errno));
+		return -1;
+	}
+	do {
+		f = fscanf (fp, "Linux version %s", kver);
+	} while (f != 1);
+	fclose(fp);
+	kver[255] = '\0';
+
+	clog(LOG_INFO, "read kernel version %s.\n", kver);
+
+	if (strstr(kver, "2.6") == kver) {
+		clog(LOG_DEBUG, "kernel version is 2.6.\n");
+		return KERNEL_VERSION_26;
+	} else if (strstr(kver, "2.4") == kver) {
+		clog(LOG_DEBUG, "kernel version is 2.4.\n");
+		return KERNEL_VERSION_24;
+	} else {
+		clog(LOG_WARNING, "Unknown kernel version, assuming a 2.6 kernel.\n");
+		return KERNEL_VERSION_26;
+	}
+
+}
 
-static struct rule *current_rule;
-static struct profile *current_profile;
-static int force_reinit = 0;
-static int force_exit = 0;
-static int timer_expired = 1; /* expired in order to run on the first loop */
-static int cpufreqd_mode = ARG_DYNAMIC; /* operation mode (manual / dynamic) */
 
 /*
  * Evaluates the full rule and returns the percentage score
  * for it.
  */
 static unsigned int rule_score(struct rule *rule) {
-	unsigned int hits = 0;
+	unsigned int hits = 0, directives = 0;
 	struct directive *d = NULL;
 	
 	/* call plugin->evaluate for each rule */
 	LIST_FOREACH_NODE(node, &rule->directives) {
 		d = (struct directive *) node->content;
 		/* compute scores for rules and keep the highest */
-		if (d->keyword->evaluate != NULL && d->keyword->evaluate(d->obj) == MATCH) {
-			hits++;
-			clog(LOG_DEBUG, "Rule \"%s\": %s matches.\n", rule->name,
-					d->keyword->word);
+		if (d->keyword->evaluate != NULL) {
+			directives++;
+			if (d->keyword->evaluate(d->obj) == MATCH) {
+				hits++;
+				clog(LOG_DEBUG, "Rule \"%s\": %s matches.\n", rule->name,
+						d->keyword->word);
+			}
 		}
 	} /* end foreach rule entry */
-	return hits + (100 * hits / rule->directives_count);
+	if (directives > 0)
+		return hits + (100 * hits / directives);
+	clog(LOG_INFO, "No evaulatable directives in Rule \"%s\".\n", rule->name);
+	return 0;
 }
 
 /*  struct rule *update_rule_scores(struct LIST *rules)
@@ -140,11 +183,11 @@ static int cpufreqd_set_profile (struct profile *old, struct profile *new) {
 				old!=NULL? &old->policy : NULL, &new->policy);
 	}
 	/* int cpufreq_set_policy(unsigned int cpu, struct cpufreq_policy *policy) */ 
-	for (i=0; i<configuration.cpu_num; i++) {
+	for (i = 0; i < cpufreqd_info->cpus; i++) {
 		if (cpufreq_set_policy(i, &(new->policy)) == 0) {
 			clog(LOG_NOTICE, "Profile \"%s\" set for cpu%d\n", new->name, i);
 			/* double check if everything is OK (configurable) */
-			if (configuration.double_check) {
+			if (configuration->double_check) {
 				struct cpufreq_policy *check = NULL;
 				check = cpufreq_get_policy(i);
 				if (check->max != new->policy.max || check->min != new->policy.min ||
@@ -178,9 +221,38 @@ static int cpufreqd_set_profile (struct profile *old, struct profile *new) {
 				old!=NULL? &old->policy : NULL, &new->policy);
 	}
 	current_profile = new;
+	cpufreqd_info->cur_policy = &new->policy;
 	return 0;
 }
 
+static int set_cpufreqd_runmode(int mode) {
+	if (mode == MODE_DYNAMIC) {
+		struct itimerval new_timer;
+		new_timer.it_interval.tv_usec = configuration->poll_intv.tv_usec;
+		new_timer.it_interval.tv_sec = configuration->poll_intv.tv_sec;
+		new_timer.it_value.tv_usec = configuration->poll_intv.tv_usec;
+		new_timer.it_value.tv_sec = configuration->poll_intv.tv_sec;
+		/* set next alarm */
+		if (setitimer(ITIMER_REAL, &new_timer, 0) < 0) {
+			clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
+			return errno;
+		}
+		timer_expired = 1;
+	} 
+	else if (mode == MODE_MANUAL) {
+		/* reset alarm */
+		if (setitimer(ITIMER_REAL, NULL, 0) < 0) {
+			clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
+			return errno;
+		}
+	}
+	else {
+		clog(LOG_WARNING, "Unknown mode %d\n", mode);
+		return EINVAL;
+	}
+	cpufreqd_mode = mode;
+	return 0;
+}
 
 /*  int read_args (int argc, char *argv[])
  *  Reads command line arguments
@@ -192,39 +264,43 @@ static int read_args (int argc, char *argv[]) {
 		{ "version", 0, 0, 'v' }, 
 		{ "file", 1, 0, 'f' },
 		{ "no-daemon", 0, 0, 'D' },
+		{ "manual", 0, 0, 'm' },
 		{ "verbosity", 1, 0, 'V' },
 		{ 0, 0, 0, 0 }, 
 	};
 	int ch,option_index = 0;
 
-	while ((ch = getopt_long(argc, argv, "hvf:DV:", long_options, &option_index)) != -1) {
+	while ((ch = getopt_long(argc, argv, "hvf:DmV:", long_options, &option_index)) != -1) {
 		switch (ch) {
 		case '?':
 		case 'h':
-			configuration.print_help = 1;
+			configuration->print_help = 1;
 			return 0;
 		case 'v':
-			configuration.print_version = 1;
+			configuration->print_version = 1;
 			return 0;
 		case 'f':
-			if (realpath(optarg, configuration.config_file) == NULL) {
+			if (realpath(optarg, configuration->config_file) == NULL) {
 				clog(LOG_ERR, "Error reading command line argument (%s: %s).\n",
 						optarg, strerror(errno));
 				return -1;
 			}
-			clog(LOG_DEBUG, "Using configuration file at %s\n", configuration.config_file);
+			clog(LOG_DEBUG, "Using configuration file at %s\n", configuration->config_file);
 			break;
 		case 'D':
-			configuration.no_daemon = 1;
+			configuration->no_daemon = 1;
 			break;
+		case 'm':
+			cpufreqd_mode = MODE_MANUAL;
+			return 0;
 		case 'V':
-			configuration.log_level = atoi(optarg);
-			if (configuration.log_level>7) {
-				configuration.log_level = 7;
-			} else if (configuration.log_level<0) {
-				configuration.log_level = 0;
+			configuration->log_level = atoi(optarg);
+			if (configuration->log_level>7) {
+				configuration->log_level = 7;
+			} else if (configuration->log_level<0) {
+				configuration->log_level = 0;
 			}
-			configuration.log_level_overridden = 1;
+			configuration->log_level_overridden = 1;
 			break;
 		default:
 			break;
@@ -238,8 +314,8 @@ static int read_args (int argc, char *argv[]) {
  */
 static void print_version(const char *me) {
 	printf("%s version "__CPUFREQD_VERSION__".\n", me);
-	printf("Copyright 2002,2003,2004 Mattia Dongili <"__CPUFREQD_MAINTAINER__">\n"
-	       "                         George Staikos <staikos@0wned.org>\n");
+	printf("Copyright 2002-2006 Mattia Dongili <"__CPUFREQD_MAINTAINER__">\n"
+	       "                    George Staikos <staikos@0wned.org>\n");
 }
 
 /*  void print_help(const char *me)
@@ -251,6 +327,7 @@ static void print_help(const char *me) {
 			"  -v, --version                display version information and exit\n"
 			"  -f, --file                   config file (default: "CPUFREQD_CONFIG")\n"
 			"  -D, --no-daemon              stay in foreground and print log to stdout (used to debug)\n"
+			"  -m, --manual                 start in manual mode (ignored if the enable_remote is 0)\n"
 			"  -V, --verbosity              verbosity level from 0 (less verbose) to 7 (most verbose)\n"
 			"\n"
 			"Report bugs to Mattia Dongili <malattia@linux.it>.\n", me);
@@ -288,6 +365,15 @@ static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *curre
 	struct rule *best_rule = NULL;
 	struct directive *d;
 	
+	/* update timestamp */
+	if (gettimeofday(&cpufreqd_info->timestamp, NULL) < 0) {
+		clog(LOG_ERR, "Couldn't read current time: %s\n", strerror(errno));
+	} else {
+		clog(LOG_DEBUG, "Current time is: %lu::%lu\n",
+				cpufreqd_info->timestamp.tv_sec,
+				cpufreqd_info->timestamp.tv_usec);
+	}
+
 	update_plugin_states(&conf->plugins);
 	best_rule = update_rule_scores(&conf->rules);
 
@@ -311,9 +397,10 @@ static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *curre
 		clog(LOG_DEBUG, "New Rule (\"%s\"), applying.\n", 
 				best_rule->name);
 		/* pre change event */
-		if (current != NULL && best_rule->directives.first != NULL) {
+		if (best_rule->directives.first != NULL) {
 			TRIGGER_EVENT(rule_pre_change, &best_rule->directives, d,
-					&current->prof->policy, &best_rule->prof->policy);
+					current != NULL ? &current->prof->policy : NULL,
+					&best_rule->prof->policy);
 		}
 
 		/* change frequency */
@@ -327,9 +414,10 @@ static struct rule *cpufreqd_loop(struct cpufreqd_conf *conf, struct rule *curre
 		}
 
 		/* post change event */
-		if (current != NULL && best_rule->directives.first != NULL) {
+		if (best_rule->directives.first != NULL) {
 			TRIGGER_EVENT(rule_post_change, &best_rule->directives, d,
-					&current->prof->policy, &best_rule->prof->policy);
+					current != NULL ? &current->prof->policy : NULL,
+					&best_rule->prof->policy);
 		}
 
 	} else {
@@ -391,11 +479,21 @@ static void execute_command(int sock, struct cpufreqd_conf *conf) {
 				break;
 			case CMD_SET_MODE:
 				clog(LOG_DEBUG, "CMD_SET_MODE\n");
+				set_cpufreqd_runmode((int)REMOTE_ARG(command));
+#if 0
 				cpufreqd_mode = REMOTE_ARG(command);
+				if (cpufreqd_mode == MODE_MANUAL) {
+					/* reset alarm */
+					if (setitimer(ITIMER_REAL, NULL, 0) < 0)
+						clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
+				} else {
+					timer_expired = 1;
+				}
+#endif
 				break;
 			case CMD_SET_PROFILE:
 				clog(LOG_DEBUG, "CMD_SET_PROFILE\n");
-				if (cpufreqd_mode == ARG_DYNAMIC) {
+				if (cpufreqd_mode == MODE_DYNAMIC) {
 					clog(LOG_ERR, "Couldn't set profile while running "
 							"in DYNAMIC mode.\n");
 					break;
@@ -432,6 +530,11 @@ static void execute_command(int sock, struct cpufreqd_conf *conf) {
 	}
 }
 
+
+struct cpufreqd_info * get_cpufreqd_info (void) {
+	return cpufreqd_info;
+}
+
 /*
  *  main !
  *  Let's go
@@ -441,12 +544,27 @@ int main (int argc, char *argv[]) {
 	struct sigaction signal_action;
 	sigset_t old_sigmask;
 	fd_set rfds;
+#if 0
 	struct itimerval new_timer;
-
+#endif
 	unsigned int i = 0;
 	int cpufreqd_sock = -1, peer_sock = -1; /* input pipe */
 	char dirname[MAX_PATH_LEN];
 	int ret = 0;
+	
+	configuration = malloc(sizeof(struct cpufreqd_conf));
+	if (configuration == NULL) {
+		ret = ENOMEM;
+		goto out;
+	}
+	memcpy(configuration, &default_configuration, sizeof(struct cpufreqd_conf));
+
+	cpufreqd_info = malloc(sizeof(struct cpufreqd_info));
+	if (cpufreqd_info == NULL) {
+		ret = ENOMEM;
+		goto out;
+	}
+	memset(cpufreqd_info, 0, sizeof(struct cpufreqd_info));
 
 	/* 
 	 *  check perms
@@ -454,7 +572,7 @@ int main (int argc, char *argv[]) {
 #if 1
 	if (geteuid() != 0) {
 		cpufreqd_log(LOG_CRIT, "%s: must be run as root.\n", argv[0]);
-		ret = 1;
+		ret = EACCES;
 		goto out;
 	}
 #endif
@@ -464,14 +582,14 @@ int main (int argc, char *argv[]) {
 	 */
 	if (read_args(argc, argv)!=0) {
 		cpufreqd_log(LOG_CRIT, "Unable parse command line parameters, exiting.\n");
-		ret = 1;
+		ret = EINVAL;
 		goto out;
 	}
-	if (configuration.print_help) {
+	if (configuration->print_help) {
 		print_help(argv[0]);
 		goto out;
 	}
-	if (configuration.print_version) {
+	if (configuration->print_version) {
 		print_version(argv[0]);
 		goto out;
 	}
@@ -500,93 +618,100 @@ int main (int argc, char *argv[]) {
 	signal_action.sa_handler = pipe_handler;
 	sigaction(SIGPIPE, &signal_action, 0);
 
+	/* read kernel version */
+	cpufreqd_info->kernel_version = get_kversion();
+	
 	/*
 	 *  read how many cpus are available here
 	 */
-	configuration.cpu_num = get_cpu_num();
+	cpufreqd_info->cpus = get_cpu_num();
 
 	/*
 	 *  find cpufreq information about each cpu
 	 */
-	if ((configuration.sys_info = malloc(configuration.cpu_num * sizeof(struct cpufreq_sys_info))) == NULL) {
+	cpufreqd_info->sys_info = calloc(1, cpufreqd_info->cpus * sizeof(struct cpufreq_sys_info));
+	if (cpufreqd_info->sys_info == NULL) {
 		clog(LOG_CRIT, "Unable to allocate memory (%s), exiting.\n", strerror(errno));
-		ret = 1;
+		ret = ENOMEM;
 		goto out;
 	}
-	for (i=0; i<configuration.cpu_num; i++) {
-		(configuration.sys_info+i)->affected_cpus = cpufreq_get_affected_cpus(i);
-		(configuration.sys_info+i)->governors = cpufreq_get_available_governors(i);
-		(configuration.sys_info+i)->frequencies = cpufreq_get_available_frequencies(i);
+	for (i = 0; i < cpufreqd_info->cpus; i++) {
+		(cpufreqd_info->sys_info+i)->affected_cpus = cpufreq_get_affected_cpus(i);
+		(cpufreqd_info->sys_info+i)->governors = cpufreq_get_available_governors(i);
+		(cpufreqd_info->sys_info+i)->frequencies = cpufreq_get_available_frequencies(i);
 	}
 
 	/* SMP: with different speed cpus */
-	if ((configuration.limits = malloc(configuration.cpu_num * sizeof(struct cpufreq_limits))) == NULL) {
+	cpufreqd_info->limits = calloc(1, cpufreqd_info->cpus * sizeof(struct cpufreq_limits));
+	if (cpufreqd_info->limits == NULL) {
 		clog(LOG_CRIT, "Unable to allocate memory (%s), exiting.\n", strerror(errno));
-		ret = 1;
+		ret = ENOMEM;
 		goto out_sys_info;
 	}
-	memset(configuration.limits, 0, configuration.cpu_num * sizeof(struct cpufreq_limits));
-	for (i=0; i<configuration.cpu_num; i++) {
+	for (i = 0; i < cpufreqd_info->cpus; i++) {
 		/* if one of the probes fails remove all the others also */
-		if (cpufreq_get_hardware_limits(i,
-					&((configuration.limits+i)->min), &((configuration.limits+i)->max))!=0) {
+		struct cpufreq_limits *tmp_lim = cpufreqd_info->limits+i;
+		if (cpufreq_get_hardware_limits(i, &tmp_lim->min, &tmp_lim->max) != 0) {
 			/* TODO: if libcpufreq fails try to read /proc/cpuinfo
 			 * and warn about this not being reliable
 			 */
 			clog(LOG_WARNING, "Unable to get hardware frequency limits for CPU%d.\n", i);
-			free(configuration.limits);
-			configuration.limits = NULL;
+			free(cpufreqd_info->limits);
+			cpufreqd_info->limits = NULL;
 			break;
 		} else {
 			clog(LOG_INFO, "Limits for cpu%d: MIN=%lu - MAX=%lu\n", i, 
-					(configuration.limits+i)->min, (configuration.limits+i)->max);
+					tmp_lim->min, tmp_lim->max);
 		}
 	}
 
 	/*
 	 *  daemonize if necessary
 	 */
-	if (configuration.no_daemon==0 && daemonize()!=0) {
+	if (configuration->no_daemon==0 && daemonize()!=0) {
 		clog(LOG_CRIT, "Unable to go background, exiting.\n");
-		ret = 1;
+		ret = ECHILD;
 		goto out_limits;
 	}
 
 cpufreqd_start:
 
-	if (init_configuration(&configuration) < 0) {
-		clog(LOG_CRIT, "Unable to parse config file: %s\n", configuration.config_file);
-		ret = 1;
+	if (init_configuration(configuration, cpufreqd_info) < 0) {
+		clog(LOG_CRIT, "Unable to parse config file: %s\n", configuration->config_file);
+		ret = EINVAL;
 		goto out_config_read;
 	}
 
 	/* setup UNIX socket if necessary */
-	if (configuration.enable_remote) {
+	if (configuration->enable_remote) {
 		dirname[0] = '\0';
-		if (create_temp_dir(dirname, configuration.remote_gid) == NULL) {
+		if (create_temp_dir(dirname, configuration->remote_gid) == NULL) {
 			clog(LOG_ERR, "Couldn't create temporary directory %s\n", dirname);
 			cpufreqd_sock = -1;
-		} else if ((cpufreqd_sock = open_unix_sock(dirname, configuration.remote_gid)) == -1) {
+		} else if ((cpufreqd_sock = open_unix_sock(dirname, configuration->remote_gid)) == -1) {
 			delete_temp_dir(dirname);
 			clog(LOG_ERR, "Couldn't open socket, remote controls disabled\n");
 		} else {
 			clog(LOG_INFO, "Remote controls enabled\n");
+			if (cpufreqd_mode == MODE_MANUAL)
+				clog(LOG_INFO, "Starting in manual mode\n");
 		}
-	}
+	} else
+		cpufreqd_mode = MODE_DYNAMIC;
 
 	/* Validate plugins, if none left exit.... */
-	if (validate_plugins(&configuration.plugins) == 0) {
+	if (validate_plugins(&configuration->plugins) == 0) {
 		cpufreqd_log(LOG_CRIT, "Hey! all the plugins I loaded are useless, "
 				"maybe your configuration needs some rework.\n"
 				"Exiting.\n");
-		ret = 1;
+		ret = EINVAL;
 		goto out_socket;
 	}
 
 	/* write pidfile */
-	if (write_cpufreqd_pid(configuration.pidfile) < 0) {
-		clog(LOG_CRIT, "Unable to write pid file: %s\n", configuration.pidfile);
-		ret = 1;
+	if (write_cpufreqd_pid(configuration->pidfile) < 0) {
+		clog(LOG_CRIT, "Unable to write pid file: %s\n", configuration->pidfile);
+		ret = EINVAL;
 		goto out_socket;
 	}
 
@@ -607,6 +732,7 @@ cpufreqd_start:
 		sigprocmask(SIG_BLOCK, &signal_action.sa_mask, &old_sigmask);
 	}
 	
+	set_cpufreqd_runmode(cpufreqd_mode);
 	/*
 	 *  Looooooooop
 	 */
@@ -615,19 +741,24 @@ cpufreqd_start:
 		 * Run the system scan and rule selection and set timer
 		 * if running in DYNAMIC mode AND the timer is expired
 		 */
-		if (cpufreqd_mode == ARG_DYNAMIC && timer_expired) {
-			timer_expired = 0;
-			new_timer.it_interval.tv_usec = 0;
-			new_timer.it_interval.tv_sec = 0;
-			new_timer.it_value.tv_usec = configuration.poll_intv.tv_usec;
-			new_timer.it_value.tv_sec = configuration.poll_intv.tv_sec;
-			current_rule = cpufreqd_loop(&configuration, current_rule);
+		if (cpufreqd_mode == MODE_DYNAMIC && timer_expired) {
+#if 0
+			new_timer.it_interval.tv_usec = configuration->poll_intv.tv_usec;
+			new_timer.it_interval.tv_sec = configuration->poll_intv.tv_sec;
+			new_timer.it_value.tv_usec = configuration->poll_intv.tv_usec;
+			new_timer.it_value.tv_sec = configuration->poll_intv.tv_sec;
+#endif
+			current_rule = cpufreqd_loop(configuration, current_rule);
+#if 0
 			/* set next alarm */
 			if (setitimer(ITIMER_REAL, &new_timer, 0) < 0) {
+				ret = errno;
 				clog(LOG_CRIT, "Couldn't set timer: %s\n", strerror(errno));
-				ret = 1;
 				break;
 			}
+#endif
+			/* can safely reset the expired flag now */
+			timer_expired = 0;
 		}
 
 		/* if the socket opened successfully */
@@ -636,16 +767,16 @@ cpufreqd_start:
 			FD_ZERO(&rfds);
 			FD_SET(cpufreqd_sock, &rfds);
 			
-			if (!timer_expired || cpufreqd_mode == ARG_MANUAL) {
+			if (!timer_expired || cpufreqd_mode == MODE_MANUAL) {
 				switch (pselect(cpufreqd_sock+1, &rfds, NULL, NULL, NULL, &old_sigmask)) {
 					case 0:
 						/* timed out. check to see if things have changed */
-						/* should never happen actually */
+						/* will never happen actually... */
 						break;
 					case -1:
-						/* caused by SIGALARM (mostly) */
+						/* caused by SIGALARM (mostly) log if not so */
 						if (errno != EINTR)
-							clog(LOG_NOTICE, "poll(): %s.\n", strerror(errno));
+							clog(LOG_NOTICE, "pselect(): %s.\n", strerror(errno));
 						break;
 					case 1:
 						/* somebody tried to contact us. see what he wants */
@@ -654,7 +785,7 @@ cpufreqd_start:
 							clog(LOG_ALERT, "Unable to accept connection: "
 									" %s\n", strerror(errno));
 						}
-						execute_command(peer_sock, &configuration);
+						execute_command(peer_sock, configuration);
 						close(peer_sock);
 						peer_sock = -1;
 						break;
@@ -675,7 +806,7 @@ cpufreqd_start:
 	/*
 	 * Clean pidfile
 	 */
-	clear_cpufreqd_pid(configuration.pidfile);
+	clear_cpufreqd_pid(configuration->pidfile);
 
 	/* close socket */
 out_socket:
@@ -688,7 +819,7 @@ out_socket:
 	 *  Free configuration structures
 	 */
 out_config_read:
-	free_configuration(&configuration);
+	free_configuration(configuration);
 	if (force_reinit && !force_exit) {
 		force_reinit = 0;
 		cpufreqd_log(LOG_INFO, "Restarting cpufreqd\n");
@@ -696,25 +827,29 @@ out_config_read:
 	}
 
 out_limits:
-	if (configuration.limits != NULL)
-		free(configuration.limits);
+	if (cpufreqd_info->limits != NULL)
+		free(cpufreqd_info->limits);
 
 out_sys_info:
-	if (configuration.sys_info != NULL) {
-		for (i=0; i<configuration.cpu_num; i++) {
-			if ((configuration.sys_info+i)->governors!=NULL)
-				cpufreq_put_available_governors((configuration.sys_info+i)->governors);
-			if ((configuration.sys_info+i)->affected_cpus!=NULL)
-				cpufreq_put_affected_cpus((configuration.sys_info+i)->affected_cpus);
-			if ((configuration.sys_info+i)->frequencies!=NULL)
-				cpufreq_put_available_frequencies((configuration.sys_info+i)->frequencies);
+	if (cpufreqd_info->sys_info != NULL) {
+		for (i=0; i<cpufreqd_info->cpus; i++) {
+			if ((cpufreqd_info->sys_info+i)->governors!=NULL)
+				cpufreq_put_available_governors((cpufreqd_info->sys_info+i)->governors);
+			if ((cpufreqd_info->sys_info+i)->affected_cpus!=NULL)
+				cpufreq_put_affected_cpus((cpufreqd_info->sys_info+i)->affected_cpus);
+			if ((cpufreqd_info->sys_info+i)->frequencies!=NULL)
+				cpufreq_put_available_frequencies((cpufreqd_info->sys_info+i)->frequencies);
 		}
-		free(configuration.sys_info);
+		free(cpufreqd_info->sys_info);
 	}
 
+out:
+	if (cpufreqd_info != NULL)
+		free(cpufreqd_info);
+	if (configuration != NULL)
+		free(configuration);
 	/*
 	 *  bye bye  
 	 */
-out:
 	return ret;
 }
